
#include "xparameters.h"


#include "stdio.h"
#include <string.h>

#include "mb_interface.h"
#include "xintc.h"
#include "xutil.h"
#include "xuartlite_l.h"
#include "xuartlite.h"
#include "stage2boot/gfxlib.c"

#define NUMBERS_COLOUR      176
typedef void (* Target_Fn) ( void ) ;

#define QUEUE_SIZE      256
static unsigned char    rx_queue[ QUEUE_SIZE ];
static unsigned         rx_queue_size = 0;
static unsigned         rx_queue_in = 0;
static unsigned         rx_queue_out = 0;
static unsigned char    tx_queue[ QUEUE_SIZE ];
static unsigned         tx_queue_size = 0;
static unsigned         tx_queue_in = 0;
static unsigned         tx_queue_out = 0;

static void TX_Queue(void)
{
    while ((tx_queue_size != 0)
    && (!(XIo_In32(XPAR_UARTLITE_0_BASEADDR + 0x8) & 0x8))) {
        unsigned char byte = tx_queue[ tx_queue_out ];
        tx_queue_out = (tx_queue_out + 1) % QUEUE_SIZE;
        tx_queue_size --;
        XIo_Out32(XPAR_UARTLITE_0_BASEADDR + 0x4, byte);
    }
}

static void My_ISR ( void * ignore )
{
    unsigned mask = 1 << XPAR_INTC_0_UARTLITE_0_VEC_ID;

	while (XIo_In32(XPAR_UARTLITE_0_BASEADDR + 0x8) & 1) {
        if (rx_queue_size >= (QUEUE_SIZE - 2)) {
            /* oh dear */
            break;
        }
        rx_queue[ rx_queue_in ] = XIo_In32(XPAR_UARTLITE_0_BASEADDR + 0x0);
        rx_queue_in = (rx_queue_in + 1) % QUEUE_SIZE;
        rx_queue_size ++;
    }
    TX_Queue();
	XIo_Out32(XPAR_INTC_0_BASEADDR + 0xc, mask); /* IAR */
}

static unsigned getbyte ( void )
{
    unsigned out = 0, ready = 0;
    do {
        microblaze_disable_interrupts();
        if (rx_queue_size == 0) {
            /* buffer empty */
        } else {
            ready = 1;
            out = rx_queue[ rx_queue_out ];
            rx_queue_out = (rx_queue_out + 1) % QUEUE_SIZE;
            rx_queue_size --;
        }
        microblaze_enable_interrupts();
    } while (!ready) ;
    return out;
}

static unsigned hasbyte ( void )
{
    unsigned ready;

    microblaze_disable_interrupts();
    ready = (rx_queue_size != 0);
    microblaze_enable_interrupts();
    return ready;
}

static void printchar ( char s )
{
    unsigned done = 0;

    do {
        microblaze_disable_interrupts();
        if (tx_queue_size < (QUEUE_SIZE - 2)) {
            done = 1;
            tx_queue[ tx_queue_in ] = (unsigned char) s;
            tx_queue_in = (tx_queue_in + 1) % QUEUE_SIZE;
            tx_queue_size ++;
            TX_Queue();
        }
        microblaze_enable_interrupts();
    } while (!done);
}

void print ( char * s )
{
    int i;
    for (i = 0; s[ i ] != '\0'; i++) {
        printchar(s[ i ]);
    }
}

void outbyte ( char s )
{
    printchar (s);
}

static const char * names [] = {
    "north",
    "east",
    "south",
    "west",
    "centre",
    "ccw",
    "cw",
    "sw state",
    NULL };

static void Start_Video ()
{
    unsigned i;
    for (i = 0; i < 64000; i++) {
        video [ i ] = 0x0; 
    }
    for (i = 0; i < 320; i++) {     /* white border */
        video [ i ] = 0x4; 
        video [ i + ( 199 * 320 ) ] = 0x4; 
    }
    for (i = 1; i < 199; i++) {
        video [ i * 320 ] = 0x4; 
        video [ ( i * 320 ) + 319 ] = 0x4; 
    }
    vga [ 0 ] = (unsigned) video ;

    Text ( 1, 2,  " Microblaze/S3ESK", 0x40 );
    Text ( 1, 10, " Stage 1 boot loader", 0x10 );
    Text ( 1, 18, " Send data now", 0x10 );
}

static void Start_Serial ()
{
    unsigned mask = 1 << XPAR_INTC_0_UARTLITE_0_VEC_ID;

    microblaze_disable_interrupts();

    /* enable interrupts on UART */
	XIo_Out32(XPAR_UARTLITE_0_BASEADDR + 0xc, 0x13);
    /* enable interrupts on intc */
	XIo_Out32(XPAR_INTC_0_BASEADDR + 0x8, mask);        /* IER */
	XIo_Out32(XPAR_INTC_0_BASEADDR + 0x1c, 0x3);        /* MER */

    microblaze_register_handler((XInterruptHandler) My_ISR, NULL);
    microblaze_enable_interrupts();
}

int main ( void )
{
    unsigned        i, size = 0 ;
    unsigned        target = 0 ;

    
    Start_Serial ();

    print ( "\r\nStage 1 boot loader: " ) ;
    Start_Video ();
    print ( "ready\r\n" ) ;

    /* Poll for switch/button state */
    do {
        unsigned y = 100;
        for (i = 0; names[ i ] != NULL; i++) {
            Text(150, y, names[ i ], 0x50 + i);
            Text_Hex(150+(9*8), y, shm[ i ], 0x32 + i);
            y += 8;
        }
        shm[ i ] = 1 << ((shm[ 5 ] - shm[ 6 ]) & 7); /* leds */
    } while (!hasbyte());

    Text ( 5, 47, "Size:   ", NUMBERS_COLOUR );

    for ( i = 0 ; i < 4 ; i ++ )
    {
        size = ( size << 8 ) | (unsigned) ( (unsigned char) getbyte () ) ;
    }
    size = size >> 2 ;
    print ("Size is ");
    printhex (size);

    Text_Hex ( 5 + (8*8), 47, size, NUMBERS_COLOUR );
    Text ( 5, 55, "Target: ", NUMBERS_COLOUR );

    target = 0 ;
    for ( i = 0 ; i < 4 ; i ++ )
    {
        target = ( target << 8 ) | (unsigned) ( (unsigned char) getbyte () ) ;
    }

    print (" Target is ");
    printhex (target);
    print ("\r\n");

    Text ( 5, 63, "Copied: ", NUMBERS_COLOUR );
    Text_Hex ( 5 + (8*8), 55, target, NUMBERS_COLOUR );

    {
        volatile unsigned * target_data = (unsigned *) target ;
        volatile unsigned char * td2 = (unsigned char *) target_data ;
        Target_Fn       target_code ;

        td2 [ 0 ] = 0x12 ;
        td2 [ 1 ] = 0x34 ;
        if (( td2 [ 0 ] != 0x12 )
        || ( td2 [ 1 ] != 0x34 ))
        {
            Text ( 5, 75, "Memory error", 0x10 );
            while (1) {}
        }

        target_code = (Target_Fn) target_data ;

        for ( i = 0 ; i < size ; i ++ )
        {
            unsigned r = ( getbyte () << 24 ) |
                    ( getbyte () << 16 ) |
                    ( getbyte () << 8 ) |
                    ( getbyte () << 0 ) ;

            target_data [ i ] = r ;
            target_data [ i + 1 ] = 0 ;

            if ( target_data [ i ] != r )
            {
                Text ( 5, 75, "Memory error", 0x10 );
                printchar ( '?' ) ;
                while ( 1 ) {}
            } else if (( i & 0x1ff ) == 0 )
            {
                Text_Hex ( 5 + (8*8), 63, i, NUMBERS_COLOUR );
            }
        }
        Text_Hex ( 5 + (8*8), 63, i, NUMBERS_COLOUR );
        Text ( 5, 75, "Launching stage 2", 0x20 );
        print ( "Launching stage 2\r\n" ) ;
        microblaze_disable_interrupts();
        target_code () ;
        while (1) {}
    }
    return 0 ;
}




