
#include "gfxlib.h"
#include "xparameters.h"

volatile unsigned * vga = (unsigned *) XPAR_PLBVGA_0_BASEADDR;
unsigned * sdram = (unsigned *)              0x8c000000;
volatile unsigned * bootloader = (unsigned *)0x8f000000;
unsigned char * video = (unsigned char *)    0x8ff00000;
volatile unsigned * shm = (unsigned *)       0x8fffff00;


#include "numbers.c"
#include "letters.c"

void Text_Char(unsigned x, unsigned y,
        unsigned ch, unsigned char colour)
{
    unsigned char * vout = & video [ x + (y * 320) ];

    for (y = 0; y < 8; y ++) {
        unsigned char cin = 0;
        if ((ch >= 48) && (ch <= 58)) {
            cin = numbers_data[ ((ch - 48) * 8) + y ];
        } else if ((ch >= 65) && (ch <= 90)) {
            cin = letters_data[ ((ch - 65) * 8) + y ];
        } else if ((ch >= 97) && (ch <= 122)) {
            cin = letters_data[ ((ch - 97) * 8) + y ];
        } 

        for (x = 0; x < 8; x ++) {
            if (cin & 0x80) {
                vout[ x ] = colour;
            } else {
                vout[ x ] = 0;
            }
            cin = cin << 1;
        }
        vout += 320;
    }
}

void Text(unsigned x, unsigned y,
        const char * ch, unsigned char colour)
{
    while (ch[ 0 ] != '\0') {
        Text_Char(x, y, (unsigned char) ch[ 0 ], colour);
        x += 8;
        ch ++;
    }
}

void Text_Hex(unsigned x, unsigned y,
        unsigned num, unsigned char colour)
{
    unsigned i , z ;

    for ( i = 0 ; i < 32 ; i += 4 )
    {
        z = ( num << i ) >> 28 ;
        if ( z < 10 )
        {
            z += '0';
        } else {
            z += 'a' - 10;
        }
        Text_Char(x, y, z, colour);
        x += 8;
    }
} 

void printhex ( unsigned x )
{
    unsigned i , y ;

    for ( i = 0 ; i < 32 ; i += 4 )
    {
        y = ( x << i ) >> 28 ;
        if ( y < 10 )
        {
            outbyte( y + '0' );
        } else {
            outbyte( y + 'a' - 10 );
        }
    }
} 

void Stop_Polling(void)
{
    shm[ 8 ] = 0x00000011; 
    shm[ 8 ] = 0xdd000011; /* special code: stop polling */
}

void Recache_Buttons(void)
{
    unsigned discard = (unsigned) shm;
    asm volatile ( 
            "wdc	%0, r0\n" 
            "addik  %0, %0, 16\n"
            "wdc	%0, r0\n" 
            "addik  %0, %0, 16\n"
            "wdc	%0, r0\n" 
                : "=&r"(discard) : "0"(discard) );
}


